package com.tzld.longarticle.recommend.server.remote.aigc;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.service.recommend.recall.RecallParam;
import lombok.extern.slf4j.Slf4j;
import okhttp3.*;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * @author dyp
 */
@Service
@Slf4j
public class AIGCWaitingPublishContentService {

    private OkHttpClient client;

    @PostConstruct
    public void init() {
        client = new OkHttpClient().newBuilder()
                .connectTimeout(5, TimeUnit.MINUTES)
                .readTimeout(5, TimeUnit.MINUTES)
                .writeTimeout(5, TimeUnit.MINUTES)
                .build();
    }

    private static final String aigcGetAllContentUrl = "http://aigc-api.cybertogether.net/aigc/publish/content/gzhWaitingPublishContent";


    public List<Content> getAllContent(RecallParam param) {
        int retryTimes = 2;
        while (retryTimes > 0) {
            List<Content> result = post(param);
            if (Objects.nonNull(result)) {
                return result;
            }
            retryTimes--;
        }
        return new ArrayList<>();
    }

    public List<Content> post(RecallParam param) {
        List<Content> result = null;
        JSONObject bodyParam = new JSONObject();
        JSONObject bodyParamParams = new JSONObject();
        bodyParamParams.put("accountId", param.getAccountId());
        bodyParamParams.put("planId", param.getPlanId());
        bodyParam.put("params", bodyParamParams);
        try {
            MediaType mediaType = MediaType.parse("application/json");
            RequestBody body = RequestBody.create(mediaType, JSONObject.toJSONString(bodyParam));
            Request request = new Request.Builder()
                    .url(aigcGetAllContentUrl)
                    .method("POST", body)
                    .addHeader("Content-Type", "application/json;charset=UTF-8")
                    .build();
            Response response = client.newCall(request).execute();
            if (response.isSuccessful()) {
                String responseContent = response.body().string();
                JSONObject jsonObject = JSONObject.parseObject(responseContent);
                if (jsonObject.getInteger("code") == 0) {
                    JSONArray data = jsonObject.getJSONArray("data");
                    result = JSONArray.parseArray(data.toJSONString(), Content.class);
                }
            }
        } catch (Exception e) {
            log.error("getAllContent error", e);
        }
        return result;
    }

    public Set<String> historyTitle() {
        return Collections.emptySet();
    }

}
