package com.tzld.longarticle.recommend.server.service.recommend.recall;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.common.cache.Cache;
import com.google.common.cache.CacheBuilder;
import com.google.common.collect.Lists;
import com.tzld.longarticle.recommend.server.common.ContentCountMonitor;
import com.tzld.longarticle.recommend.server.common.CostMonitor;
import com.tzld.longarticle.recommend.server.common.ThreadPoolFactory;
import com.tzld.longarticle.recommend.server.common.enums.LongArticleTextSimilarityStatusEnum;
import com.tzld.longarticle.recommend.server.common.enums.StatusEnum;
import com.tzld.longarticle.recommend.server.common.enums.aigc.PublishPlanInputSourceTypesEnum;
import com.tzld.longarticle.recommend.server.common.enums.longArticle.ArticleVideoAuditStatusEnum;
import com.tzld.longarticle.recommend.server.common.enums.recommend.*;
import com.tzld.longarticle.recommend.server.mapper.aigc.AigcBaseMapper;
import com.tzld.longarticle.recommend.server.mapper.crawler.ArticleMapper;
import com.tzld.longarticle.recommend.server.mapper.crawler.CrawlerBaseMapper;
import com.tzld.longarticle.recommend.server.mapper.longArticle.LongArticleBaseMapper;
import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.model.dto.ContentHisPublishArticle;
import com.tzld.longarticle.recommend.server.model.entity.aigc.CrawlerMetaArticle;
import com.tzld.longarticle.recommend.server.model.entity.aigc.ProducePlanExeRecord;
import com.tzld.longarticle.recommend.server.model.entity.aigc.ProduceTaskAtom;
import com.tzld.longarticle.recommend.server.model.entity.aigc.PublishContent;
import com.tzld.longarticle.recommend.server.model.entity.crawler.AccountAvgInfo;
import com.tzld.longarticle.recommend.server.model.entity.crawler.AccountCorrelation;
import com.tzld.longarticle.recommend.server.model.entity.crawler.Article;
import com.tzld.longarticle.recommend.server.model.entity.crawler.ArticleDetailInfo;
import com.tzld.longarticle.recommend.server.model.entity.longArticle.*;
import com.tzld.longarticle.recommend.server.model.param.TitleHisCacheParam;
import com.tzld.longarticle.recommend.server.remote.aigc.AIGCWaitingPublishContentService;
import com.tzld.longarticle.recommend.server.repository.aigc.CrawlerMetaArticleRepository;
import com.tzld.longarticle.recommend.server.repository.aigc.ProducePlanExeRecordRepository;
import com.tzld.longarticle.recommend.server.repository.aigc.PublishContentRepository;
import com.tzld.longarticle.recommend.server.repository.aigc.PublishPlanRepository;
import com.tzld.longarticle.recommend.server.repository.crawler.AccountAvgInfoRepository;
import com.tzld.longarticle.recommend.server.repository.crawler.AccountCorrelationRepository;
import com.tzld.longarticle.recommend.server.repository.crawler.ArticleDetailInfoRepository;
import com.tzld.longarticle.recommend.server.repository.crawler.ArticleRepository;
import com.tzld.longarticle.recommend.server.repository.longArticle.*;
import com.tzld.longarticle.recommend.server.service.recommend.config.AccountIndexAvgViewCountService;
import com.tzld.longarticle.recommend.server.service.recommend.recall.strategy.DefaultRecallStrategy;
import com.tzld.longarticle.recommend.server.service.recommend.score.ScoreStrategy;
import com.tzld.longarticle.recommend.server.util.CommonCollectionUtils;
import com.tzld.longarticle.recommend.server.util.DateUtils;
import com.tzld.longarticle.recommend.server.util.JSONUtils;
import com.tzld.longarticle.recommend.server.util.Md5Util;
import com.tzld.longarticle.recommend.server.util.feishu.FeishuMessageSender;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.PostConstruct;
import java.util.*;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author dyp
 */
@Service
@Slf4j
public class RecallService implements ApplicationContextAware {

    @Autowired
    ArticleRepository articleRepository;
    @Autowired
    AccountAvgInfoRepository accountAvgInfoRepository;
    @Autowired
    ArticleDetailInfoRepository articleDetailInfoRepository;
    @Autowired
    CrawlerMetaArticleRepository crawlerMetaArticleRepository;
    @Autowired
    AIGCWaitingPublishContentService aigcWaitingPublishContentService;
    @Autowired
    AccountIndexAvgViewCountService accountIndexAvgViewCountService;
    @Autowired
    CrawlerBaseMapper crawlerBaseMapper;
    @Autowired
    ArticleMapper articleMapper;
    @Autowired
    AccountCorrelationRepository accountCorrelationRepository;
    @Autowired
    ArticlePoolPromotionSourceRepository articlePoolPromotionSourceRepository;
    @Autowired
    ArticleCategoryRepository articleCategoryRepository;
    @Autowired
    PublishContentRepository publishContentRepository;
    @Autowired
    AccountCategoryRepository accountCategoryRepository;
    @Autowired
    PublishSingleVideoSourceRepository publishSingleVideoSourceRepository;
    @Autowired
    ArticleTitleHisCacheRepository articleTitleHisCacheRepository;
    @Autowired
    LongArticleBaseMapper longArticleBaseMapper;
    @Autowired
    LongArticlesTextRepository longArticlesTextRepository;
    @Autowired
    LongArticleTitleAuditRepository titleAuditRepository;
    @Autowired
    AigcBaseMapper aigcBaseMapper;
    @Autowired
    PublishPlanRepository publishPlanRepository;
    @Autowired
    ProducePlanExeRecordRepository producePlanExeRecordRepository;
    @Autowired
    AccountRepository accountRepository;

    private final Map<String, RecallStrategy> strategyMap = new HashMap<>();
    private ApplicationContext applicationContext;
    private final ExecutorService pool = ThreadPoolFactory.recallPool();

    Cache<String, Integer> textCache = CacheBuilder.newBuilder().expireAfterWrite(60 * 10, TimeUnit.SECONDS).build();

    @Value("${recall.content.his.feishu.enable:false}")
    private Boolean contentHisFeishuEnable;
    @Value("${morning.noon.fission.rate:0.64}")
    private double morningNoonFissionRate;
    @Value("${category.active.version:1}")
    private Integer activeVersion;


    @PostConstruct
    public void init() {
        Map<String, RecallStrategy> type = applicationContext.getBeansOfType(RecallStrategy.class);
        for (Map.Entry<String, RecallStrategy> entry : type.entrySet()) {
            RecallStrategy value = entry.getValue();
            strategyMap.put(value.getClass().getSimpleName(), value);
        }
    }

    public RecallResult recall(RecallParam param) {
        List<RecallResult.RecallData> results = new ArrayList<>();
        log.info("RecallParam {}", JSONUtils.toJson(param));
        List<RecallStrategy> strategies = getRecallStrategy(param);
        log.info("RecallStrategy {}", JSONUtils.toJson(CommonCollectionUtils.toList(strategies,
                s -> s.getClass().getSimpleName())));
        List<Content> content = getAllContent(param);
        if (CollectionUtils.isEmpty(content)) {
            return new RecallResult(results);
        }
        param.setContent(content);
        CountDownLatch cdl = new CountDownLatch(strategies.size());
        List<Future<RecallResult.RecallData>> recallResultFutures = new ArrayList<>();
        for (final RecallStrategy strategy : strategies) {
            Future<RecallResult.RecallData> future = pool.submit(() -> {
                try {
                    return strategy.recall(param);
                } finally {
                    cdl.countDown();
                }
            });
            recallResultFutures.add(future);
        }
        try {
            cdl.await(10000, TimeUnit.MILLISECONDS);
        } catch (InterruptedException e) {
            log.error("recall error", e);
            return null;
        }

        for (Future<RecallResult.RecallData> f : recallResultFutures) {
            try {
                RecallResult.RecallData data = f.get();
                results.add(data);
            } catch (Exception e) {
                log.error("future get error ", e);
            }
        }

        return new RecallResult(results);
    }

    private List<RecallStrategy> getRecallStrategy(RecallParam param) {
        List<RecallStrategy> strategies = new ArrayList<>();
        strategies.add(strategyMap.get(DefaultRecallStrategy.class.getSimpleName()));
        return strategies;
    }

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        this.applicationContext = applicationContext;
    }

    private List<Content> getAllContent(RecallParam param) {
        long t1 = System.currentTimeMillis();
        List<Content> content = aigcWaitingPublishContentService.getAllContent(param);
        long t2 = System.currentTimeMillis();
        CostMonitor.logCost("Recall", "GetAllContents", t2 - t1);
        ContentCountMonitor.logCount("Recall", "GetAllContents", Objects.isNull(content) ? 0 : content.size());
//            // 临时过滤文章视频不匹配content
//            filterNotMatchContent(content);
        // 过滤仅保留审核通过content
        filterAuditPassContent(content);
        ContentCountMonitor.logCount("Recall", "Filter", Objects.isNull(content) ? 0 : content.size());
        if (CollectionUtils.isEmpty(content)) {
            FeishuMessageSender.sendWebHookMessage(FeishuRobotIdEnum.RECOMMEND.getRobotId(),
                    "内容召回失败\n"
                            + "planId: " + param.getPlanId() + "\n"
                            + "ghId: " + param.getGhId() + "\n"
                            + "账号名称: " + param.getAccountName());
            return content;
        }
        long t3 = System.currentTimeMillis();
        // 标题历史均值
        setTitleAvgViewCount(content, param.getGhId(), param.getType(), param.getStrategy());
        long t4 = System.currentTimeMillis();
        CostMonitor.logCost("Recall", "SetAvgViewCount", t4 - t3);
        // 视频内容池查询抓取时间
        setVideoContent(content);
        return content;
    }

    private void filterAuditPassContent(List<Content> content) {
        if (CollectionUtils.isEmpty(content)) {
            return;
        }
        // 长文内容池审核过滤
        List<String> sourceIds = content.stream()
                .filter(o -> (Objects.equals(o.getSourceType(), PublishPlanInputSourceTypesEnum.producePlan.getVal())
                        || Objects.equals(o.getSourceType(), PublishPlanInputSourceTypesEnum.produceContent.getVal())))
                .map(Content::getSourceId).collect(Collectors.toList());
        List<LongArticleTitleAudit> titleAuditList = titleAuditRepository.getByContentIdIn(sourceIds);
        Map<String, Integer> titleAuditMap = titleAuditList.stream()
                .collect(Collectors.toMap(LongArticleTitleAudit::getContentId, LongArticleTitleAudit::getStatus));
        content.removeIf(o -> (Objects.equals(o.getSourceType(), PublishPlanInputSourceTypesEnum.producePlan.getVal())
                || Objects.equals(o.getSourceType(), PublishPlanInputSourceTypesEnum.produceContent.getVal()))
                && (Objects.isNull(titleAuditMap.get(o.getSourceId()))
                || titleAuditMap.get(o.getSourceId()) != ArticleVideoAuditStatusEnum.PASS.getCode()));
        if (CollectionUtils.isEmpty(content)) {
            return;
        }
        // 视频内容池审核过滤
        List<String> videoPoolSourceIds = content.stream()
                .filter(o -> Objects.equals(o.getSourceType(), PublishPlanInputSourceTypesEnum.longArticleVideoPoolSource.getVal()))
                .map(Content::getSourceId).collect(Collectors.toList());
        List<PublishSingleVideoSource> videoSourceList = publishSingleVideoSourceRepository.getByContentTraceIdIn(videoPoolSourceIds);
        Map<String, Integer> videoAuditMap = videoSourceList.stream()
                .collect(Collectors.toMap(PublishSingleVideoSource::getContentTraceId, PublishSingleVideoSource::getVideoPoolAuditStatus));
        content.removeIf(o -> Objects.equals(o.getSourceType(), PublishPlanInputSourceTypesEnum.longArticleVideoPoolSource.getVal())
                && (Objects.isNull(videoAuditMap.get(o.getSourceId()))
                || videoAuditMap.get(o.getSourceId()) != ArticleVideoAuditStatusEnum.PASS.getCode()));
    }

    private void filterNotMatchContent(List<Content> content) {
        if (CollectionUtils.isEmpty(content)) {
            return;
        }
        List<String> sourceIds = content.stream().map(Content::getSourceId).collect(Collectors.toList());
        Map<String, Integer> textStatusMap = new HashMap<>(textCache.getAllPresent(sourceIds));
        List<String> searchIds = content.stream()
                .filter(o -> !Objects.equals(o.getSourceType(), PublishPlanInputSourceTypesEnum.longArticleVideoPoolSource.getVal()))
                .map(Content::getSourceId).filter(o -> Objects.isNull(textStatusMap.get(o)))
                .collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(searchIds)) {
            List<LongArticlesText> textList = longArticlesTextRepository.getByContentIdIn(searchIds);
            Map<String, Integer> textMap = textList.stream()
                    .collect(Collectors.toMap(LongArticlesText::getContentId, LongArticlesText::getSimilarityStatus));
            textStatusMap.putAll(textMap);
            textCache.putAll(textMap);
        }
        content.removeIf(o -> !Objects.equals(o.getSourceType(), PublishPlanInputSourceTypesEnum.longArticleVideoPoolSource.getVal())
                && (Objects.isNull(textStatusMap.get(o.getSourceId()))
                || textStatusMap.get(o.getSourceId()) != LongArticleTextSimilarityStatusEnum.AVAILABLE.getCode()));
    }

    private void setVideoContent(List<Content> contentList) {
        List<String> contentTraceIds = contentList.stream()
                .filter(o -> PublishPlanInputSourceTypesEnum.longArticleVideoPoolSource.getVal().equals(o.getSourceType()))
                .map(Content::getSourceId).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(contentTraceIds)) {
            return;
        }
        Map<String, Content> contentMap = contentList.stream()
                .collect(Collectors.toMap(Content::getSourceId, Function.identity(), (a, b) -> a));
        List<PublishSingleVideoSource> sourceList = publishSingleVideoSourceRepository.getByContentTraceIdIn(contentTraceIds);
        Map<String, PublishSingleVideoSource> sourceMap = sourceList.stream()
                .collect(Collectors.toMap(PublishSingleVideoSource::getContentTraceId, Function.identity()));
        for (String contentTraceId : contentTraceIds) {
            Content content = contentMap.get(contentTraceId);
            PublishSingleVideoSource source = sourceMap.get(contentTraceId);
            if (Objects.isNull(source)) {
                continue;
            }
            content.setCrawlerTimestamp(source.getCrawlerTimestamp());
            ContentPoolEnum poolEnum = ContentPoolEnum.from(source.getFlowPoolLevel());
            content.setContentPoolType(poolEnum.getContentPool());
        }
    }

    private List<CrawlerMetaArticle> getByUniqueIndexIn(List<String> md5List) {
        if (CollectionUtils.isEmpty(md5List)) {
            return new ArrayList<>();
        }
        return crawlerMetaArticleRepository.getByUniqueIndexIn(md5List);
    }


    public static String generateArticleUniqueMd5(String url) {
        // Extract parts from the URL
        String biz = extractParameter(url, "biz=");
        String idx = extractParameter(url, "&idx=");
        String sn = extractParameter(url, "&sn=");

        // Combine the parts and encode them to bytes
        String urlBit = String.format("%s-%s-%s", biz, idx, sn);
        return Md5Util.encoderByMd5(urlBit);
    }

    private static String extractParameter(String url, String parameter) {
        int start = url.indexOf(parameter) + parameter.length();
        if (start == -1 || start == url.length()) {
            return "";
        }
        int end = url.indexOf("&", start);
        if (end == -1) {
            return url.substring(start);
        } else {
            return url.substring(start, end);
        }
    }

    public void setTitleAvgViewCount(List<Content> contentList, String ghId, String type, String strategy) {
        long start = System.currentTimeMillis();
        contentList.forEach(content -> content.setTitleMd5(Md5Util.encoderByMd5(content.getTitle())));
        List<String> sourceIdList = contentList.stream().map(Content::getSourceId).distinct().collect(Collectors.toList());
        // 根据sourceId查询数据库获取数据
        List<ArticleTitleHisCache> articleTitleHisCacheList = new ArrayList<>(sourceIdList.size());
        for (List<String> partition : Lists.partition(sourceIdList, 1000)) {
            articleTitleHisCacheList.addAll(articleTitleHisCacheRepository.getBySourceIdInAndType(partition, type));
        }
        Map<String, ArticleTitleHisCache> articleTitleHisCacheMap = articleTitleHisCacheList.stream()
                .collect(Collectors.toMap(ArticleTitleHisCache::getSourceId, Function.identity()));
        // sourceId 进行过滤 排除缓存中数据 重新走下方查询
        sourceIdList.removeIf(articleTitleHisCacheMap::containsKey);
        // 获取账号相关性
        List<AccountCorrelation> accountCorrelationList = accountCorrelationRepository.findByGhIdAndStatus(ghId, 1);
        Map<String, Double> accountCorrelationMap = accountCorrelationList.stream().collect(
                Collectors.toMap(AccountCorrelation::getRelGhId, AccountCorrelation::getCorrelation));
        List<Content> newCacheSaveList = new ArrayList<>();
        Set<String> newCacheSourceIdSet = new HashSet<>();
        for (Content content : contentList) {
            if (articleTitleHisCacheMap.containsKey(content.getSourceId())) {
                ArticleTitleHisCache cache = articleTitleHisCacheMap.get(content.getSourceId());
                List<ContentHisPublishArticle> hisPublishArticleList =
                        JSONArray.parseArray(cache.getHisPublishArticleList(), ContentHisPublishArticle.class);
                if (CollectionUtils.isNotEmpty(hisPublishArticleList)) {
                    for (ContentHisPublishArticle article : hisPublishArticleList) {
                        article.setCorrelation(Optional.ofNullable(accountCorrelationMap.get(article.getGhId())).orElse(0.0));
                    }
                }
                if (StringUtils.hasText(cache.getCategory())) {
                    content.setCategory(JSONArray.parseArray(cache.getCategory(), String.class));
                }
                content.setProducePlanId(cache.getPlanId());
                content.setKimiSafeScore(cache.getKimiSafeScore());
                content.setRootPublishTimestamp(cache.getRootPublishTimestamp());
                content.setHisPublishArticleList(hisPublishArticleList);
                setT0Data(content, ghId);
            } else {
                if (!newCacheSourceIdSet.contains(content.getSourceId())
                        && Arrays.asList(PublishPlanInputSourceTypesEnum.producePlan.getVal(),
                        PublishPlanInputSourceTypesEnum.produceContent.getVal(),
                        PublishPlanInputSourceTypesEnum.longArticleVideoPoolSource.getVal()
                ).contains(content.getSourceType())) {
                    newCacheSaveList.add(content);
                    newCacheSourceIdSet.add(content.getSourceId());
                }
            }
            if (Objects.isNull(content.getContentPoolType()) && RankStrategyEnum.FWH_STRATEGY.getStrategy().equals(strategy)) {
                content.setContentPoolType(ContentPoolEnum.autoArticlePoolLevel1.getContentPool());
            }
        }
        // 写入缓存
        saveArticleTitleHisCache(newCacheSaveList, type);
        log.info("setTitleAvgViewCount cost:{}", System.currentTimeMillis() - start);
    }

    private void saveArticleTitleHisCache(List<Content> saveList, String type) {
        if (CollectionUtils.isEmpty(saveList)) {
            return;
        }
        List<ArticleTitleHisCache> cacheList = new ArrayList<>();
        try {
            for (Content content : saveList) {
                ArticleTitleHisCache cache = new ArticleTitleHisCache();
                BeanUtils.copyProperties(content, cache);
                cache.setType(type);
                cache.setPlanId(content.getProducePlanId());
                cache.setChannelContentId(content.getCrawlerChannelContentId());
                if (CollectionUtils.isNotEmpty(content.getCategory())) {
                    cache.setCategory(JSONObject.toJSONString(content.getCategory()));
                }
                cache.setRootPublishTimestamp(content.getRootPublishTimestamp());
                if (CollectionUtils.isNotEmpty(content.getHisPublishArticleList())) {
                    cache.setHisPublishArticleList(JSONObject.toJSONString(content.getHisPublishArticleList()));
                }
                cache.setCreateTimestamp(System.currentTimeMillis());
                cacheList.add(cache);
            }
            List<String> sourceIds = cacheList.stream().map(ArticleTitleHisCache::getSourceId).collect(Collectors.toList());
            List<ArticleTitleHisCache> existsList = articleTitleHisCacheRepository.getBySourceIdInAndType(sourceIds, type);
            Set<String> existsIds = existsList.stream().map(ArticleTitleHisCache::getSourceId).collect(Collectors.toSet());
            cacheList = cacheList.stream().filter(o -> !existsIds.contains(o.getSourceId())).collect(Collectors.toList());
            if (CollectionUtils.isEmpty(cacheList)) {
                return;
            }
            longArticleBaseMapper.batchInsertArticleTitleHisCache(cacheList);
        } catch (Exception e) {
            log.error("saveArticleTitleHisCache error:{}", e.getMessage());
        }
    }

    public Map<String, Content> getArticleWithHistory(List<TitleHisCacheParam> paramList, String type) {
        Map<String, Content> result = new HashMap<>();
        List<String> titleMd5List = paramList.stream().map(TitleHisCacheParam::getTitleMd5).collect(Collectors.toList());
        List<String> sourceIds = paramList.stream().map(TitleHisCacheParam::getSourceId).collect(Collectors.toList());
        // 获取历史已发布文章
        List<Article> hisArticleList = new ArrayList<>();
        List<List<String>> titleMd5Partition = Lists.partition(new ArrayList<>(titleMd5List), 1000);
        for (List<String> titleMd5s : titleMd5Partition) {
            hisArticleList.addAll(articleMapper.getByTitleMd5InAndTypeEqualsAndStatusEquals(titleMd5s, type, 1));
        }
        Map<String, Map<Integer, List<Article>>> map = hisArticleList.stream()
                .collect(Collectors.groupingBy(Article::getTitle, Collectors.groupingBy(Article::getItemIndex)));
        Set<String> snList = hisArticleList.stream().map(Article::getWxSn).collect(Collectors.toSet());
        List<ArticleDetailInfo> articleDetailInfoList = new ArrayList<>();
        List<List<String>> snPartition = Lists.partition(new ArrayList<>(snList), 1000);
        for (List<String> sns : snPartition) {
            articleDetailInfoList.addAll(crawlerBaseMapper.getAllByWxSnIn(sns));
        }
        Map<String, List<ArticleDetailInfo>> articleDetailInfoMap = articleDetailInfoList.stream()
                .collect(Collectors.groupingBy(ArticleDetailInfo::getWxSn));
        // 获取历史已发布文章所属头条内容
        Set<String> ghIds = hisArticleList.stream().map(Article::getGhId).collect(Collectors.toSet());
        Set<String> appMsgIds = hisArticleList.stream().map(Article::getAppMsgId).collect(Collectors.toSet());
        List<Article> firstIndexHisArticleList = articleMapper.getByGhIdInAndAppMsgIdInAndItemIndexAndTypeEqualsAndStatusEquals(
                ghIds, appMsgIds, 1, type, 1);
        Map<String, Map<String, Article>> firstIndexHisArticleMap = firstIndexHisArticleList.stream()
                .collect(Collectors.groupingBy(Article::getGhId, Collectors.toMap(Article::getAppMsgId, o -> o, (a, b) -> a)));
        // 获取发布账号 位置历史均值
        List<AccountAvgInfo> accountAvgInfoList = crawlerBaseMapper.getAllByGhIdIn(ghIds);
        Map<String, Map<String, Map<String, AccountAvgInfo>>> accountAvgInfoIndexMap = accountAvgInfoList.stream()
                .filter(o -> Objects.nonNull(o.getReadAvg()) && o.getReadAvg() > 0 && o.getFans() > 1000)
                .collect(Collectors.groupingBy(AccountAvgInfo::getGhId, Collectors.groupingBy(AccountAvgInfo::getUpdateTime,
                        Collectors.toMap(AccountAvgInfo::getPosition, o -> o))));
        Map<String, Map<String, AccountAvgInfo>> firstIndexAvgInfoMap = accountAvgInfoList.stream()
                .filter(o -> "1".equals(o.getPosition()))
                .filter(o -> Objects.nonNull(o.getReadAvg()) && o.getReadAvg() > 0 && o.getFans() > 1000)
                .collect(Collectors.groupingBy(AccountAvgInfo::getGhId, Collectors.toMap(AccountAvgInfo::getUpdateTime, o -> o)));
        List<AccountCategory> accountCategoryList = accountCategoryRepository.getByStatusAndVersion(StatusEnum.ONE.getCode(), activeVersion);
        Map<String, JSONObject> accountCategoryMap = accountCategoryList.stream().filter(o -> StringUtils.hasText(o.getCategoryMap()))
                .collect(Collectors.toMap(AccountCategory::getGhId, o -> JSONObject.parseObject(o.getCategoryMap())));

        // 获取品类
        List<String> channelContentIds = paramList.stream().map(TitleHisCacheParam::getCrawlerChannelContentId)
                .filter(StringUtils::hasText).collect(Collectors.toList());
        // 查询晋升rootProduceContentId
        Map<String, ArticlePoolPromotionSource> sourceMap = new HashMap<>();
        Map<String, PublishContent> publishContentMap = new HashMap<>();
        if (CollectionUtils.isNotEmpty(channelContentIds)) {
            List<ArticlePoolPromotionSource> sourceList = articlePoolPromotionSourceRepository
                    .getByChannelContentIdInAndStatusAndDeleted(channelContentIds,
                            ArticlePoolPromotionSourceStatusEnum.FINISH.getCode(), 0);
            sourceMap = sourceList.stream()
                    .collect(Collectors.toMap(ArticlePoolPromotionSource::getChannelContentId, Function.identity()));
            List<String> publishContentIds = sourceList.stream().
                    map(ArticlePoolPromotionSource::getRootPublishContentId).collect(Collectors.toList());
            List<PublishContent> publishContentList = publishContentRepository.getByIdIn(publishContentIds);
            publishContentMap = publishContentList.stream()
                    .collect(Collectors.toMap(PublishContent::getId, Function.identity()));
        }
        // 根据produceContentId查询category
        List<ArticleCategory> articleCategoryList = articleCategoryRepository.getByStatusAndVersion(ArticleCategoryStatusEnum.SUCCESS.getCode(), activeVersion);
        Map<String, ArticleCategory> categoryMap = articleCategoryList.stream()
                .collect(Collectors.toMap(ArticleCategory::getProduceContentId, Function.identity()));
        Map<String, ArticleCategory> coldStartCategoryMap = articleCategoryList.stream()
                .filter(o -> Objects.nonNull(o.getChannelContentId()))
                .collect(Collectors.toMap(ArticleCategory::getChannelContentId, Function.identity(), (a, b) -> a));
        Map<String, ArticleCategory> titleCategoryMap = articleCategoryList.stream()
                .collect(Collectors.toMap(ArticleCategory::getTitleMd5, Function.identity(), (a, b) -> a));
        // 获取生成计划
        List<ProducePlanExeRecord> planExeRecordList = producePlanExeRecordRepository.getByPlanExeIdIn(sourceIds);
        Map<String, ProducePlanExeRecord> planExeRecordMap = planExeRecordList.stream()
                .collect(Collectors.toMap(ProducePlanExeRecord::getPlanExeId, Function.identity()));
        // 根据sourceId查询kimiSafeScore
        List<ProduceTaskAtom> safeScoreList = aigcBaseMapper.getProduceScoreByContentId(sourceIds);
        Map<String, ProduceTaskAtom> safeScoreMap = safeScoreList.stream().filter(o -> StringUtils.hasText(o.getOutput()))
                .collect(Collectors.toMap(ProduceTaskAtom::getPlanExeId, o -> o));
        List<Account> accountList = accountRepository.getByStatus(1);
        Map<String, Account> accountMap = accountList.stream()
                .collect(Collectors.toMap(Account::getGhId, Function.identity()));

        for (TitleHisCacheParam cacheParam : paramList) {
            Content res = new Content();
            // 设置品类
            ArticleCategory category = categoryMap.get(cacheParam.getSourceId());
            if (Objects.isNull(category)) {
                category = coldStartCategoryMap.get(cacheParam.getCrawlerChannelContentId());
            }
            if (Objects.isNull(category)) {
                category = titleCategoryMap.get(cacheParam.getTitleMd5());
            }
            if (Objects.nonNull(category)) {
                res.setCategory(Collections.singletonList(category.getCategory()));
            }
            // 设置kimiSafeScore
            ProduceTaskAtom atom = safeScoreMap.get(cacheParam.getSourceId());
            if (Objects.nonNull(atom)) {
                String safeScore = atom.getOutput();
                if (StringUtils.hasText(safeScore)) {
                    safeScore = safeScore.replaceAll("\n", "");
                    if (safeScore.length() == 1) {
                        res.setKimiSafeScore(safeScore);
                    }
                }
            }
            // 设置生成计划
            ProducePlanExeRecord exeRecord = planExeRecordMap.get(cacheParam.getSourceId());
            if (Objects.nonNull(exeRecord)) {
                res.setProducePlanId(exeRecord.getPlanId());
            }
            // 溯源查找源发布时间
            ArticlePoolPromotionSource source = sourceMap.get(cacheParam.getCrawlerChannelContentId());
            if (Objects.nonNull(source) && Objects.nonNull(source.getRootProduceContentId())) {
                PublishContent publishContent = publishContentMap.get(source.getRootPublishContentId());
                if (Objects.nonNull(publishContent)) {
                    res.setRootPublishTimestamp(publishContent.getPublishTimestamp());
                }
            }
            // 设置历史表现
            List<Article> hisArticles = new ArrayList<>();
            Map<Integer, List<Article>> indexArticleMap = map.get(cacheParam.getTitle());
            if (Objects.isNull(indexArticleMap)) {
                indexArticleMap = map.get(cacheParam.getCrawlerTitle());
            } else if (!cacheParam.getTitle().equals(cacheParam.getCrawlerTitle())) {
                Map<Integer, List<Article>> crawlerTitleIndexArticleMap = map.get(cacheParam.getCrawlerTitle());
                if (Objects.nonNull(crawlerTitleIndexArticleMap)) {
                    for (Map.Entry<Integer, List<Article>> entry : crawlerTitleIndexArticleMap.entrySet()) {
                        if (indexArticleMap.containsKey(entry.getKey())) {
                            List<Article> articleList = indexArticleMap.get(entry.getKey());
                            articleList.addAll(entry.getValue());
                            indexArticleMap.put(entry.getKey(), articleList);
                        } else {
                            indexArticleMap.put(entry.getKey(), entry.getValue());
                        }
                    }
                }
            }
            if (Objects.nonNull(indexArticleMap) && !indexArticleMap.isEmpty()) {
                for (Map.Entry<Integer, List<Article>> indexArticleEntry : indexArticleMap.entrySet()) {
                    List<Article> indexArticleList = indexArticleEntry.getValue();
                    hisArticles.addAll(indexArticleList);
                }
            }
            res.setHisPublishArticleList(new ArrayList<>());
            for (Article hisArticle : hisArticles) {
                if (ScoreStrategy.hisContentLateFilter(hisArticle.getPublishTimestamp())) {
                    continue;
                }
                // 历史表现 文章品类如果与历史发布账号负相关 则过滤，不计算该历史发布表现
                JSONObject categoryWeightMap = accountCategoryMap.get(hisArticle.getGhId());
                if (Objects.nonNull(categoryWeightMap) && CollectionUtils.isNotEmpty(cacheParam.getCategory())) {
                    String hisCategory = cacheParam.getCategory().get(0);
                    if (categoryWeightMap.containsKey(hisCategory)) {
                        double weight = categoryWeightMap.getDoubleValue(hisCategory);
                        if (weight < 0) {
                            continue;
                        }
                    }
                }
                ContentHisPublishArticle article = new ContentHisPublishArticle();
                BeanUtils.copyProperties(hisArticle, article);
                Account account = accountMap.get(article.getGhId());
                if (Objects.nonNull(account)) {
                    article.setAccountType(account.getType());
                }
                article.setViewCount(hisArticle.getShowViewCount());
                article.setWxSn(hisArticle.getWxSn());
                article.setArticleDetailInfoList(articleDetailInfoMap.get(hisArticle.getWxSn()));
                // 设置账号位置阅读均值
                int avgViewCount = 0;
                double readAvgCiUpper = 0.0;
                Map<String, Map<String, AccountAvgInfo>> dateAvgMap = accountAvgInfoIndexMap.get(hisArticle.getGhId());
                String hisPublishDate = DateUtils.timestampToYMDStr(article.getPublishTimestamp(), "yyyy-MM-dd");
                if (Objects.nonNull(dateAvgMap)) {
                    List<String> avgMapDateList = new ArrayList<>(dateAvgMap.keySet());
                    hisPublishDate = DateUtils.findNearestDate(avgMapDateList, hisPublishDate, "yyyy-MM-dd");
                    Map<String, AccountAvgInfo> indexMap = dateAvgMap.get(hisPublishDate);
                    if (Objects.nonNull(indexMap) && indexMap.containsKey(hisArticle.getItemIndex().toString())) {
                        article.setInnerAccount(true);
                        avgViewCount = Optional.ofNullable(indexMap.get(hisArticle.getItemIndex().toString()).getReadAvg())
                                .orElse(0.0).intValue();
                        readAvgCiUpper = Optional.ofNullable(indexMap.get(hisArticle.getItemIndex().toString()).getReadAvgCiUpper())
                                .orElse(0.0).intValue();
                    } else {
                        if (ArticleTypeEnum.QUNFA.getVal().equals(type)) {
                            log.error("历史表现阅读均值获取失败 ghId:{} accountName:{} date:{} index:{}",
                                    hisArticle.getGhId(), hisArticle.getAccountName(), hisPublishDate,
                                    hisArticle.getItemIndex());
//                            FeishuMessageSender.sendWebHookMessage(FeishuRobotIdEnum.RECOMMEND.getRobotId(),
//                                    "历史表现阅读均值获取失败\n"
//                                            + "ghId: " + hisArticle.getGhId() + "\n"
//                                            + "账号名称: " + hisArticle.getAccountName() + "\n"
//                                            + "日期: " + hisPublishDate + "\n"
//                                            + "位置: " + hisArticle.getItemIndex());
                        }
                    }
                }
                article.setAvgViewCount(avgViewCount);
                article.setReadAvgCiUpper(readAvgCiUpper);
                if (Objects.nonNull(article.getAvgViewCount()) && article.getAvgViewCount() > 0
                        && Objects.nonNull(article.getViewCount())) {
                    article.setViewCountRate((article.getViewCount() * 1.0) / article.getAvgViewCount());
                }
                // 设置头条阅读均值
                AccountAvgInfo firstIndexAvgInfo = null;
                Map<String, AccountAvgInfo> firstIndexDateAvgInfo = firstIndexAvgInfoMap.get(hisArticle.getGhId());
                if (Objects.nonNull(firstIndexDateAvgInfo)) {
                    firstIndexAvgInfo = firstIndexDateAvgInfo.get(hisPublishDate);
                    if (Objects.nonNull(firstIndexAvgInfo)) {
                        article.setFans(firstIndexAvgInfo.getFans());
                    }
                }
                Map<String, Article> firstIndexArticle = firstIndexHisArticleMap.get(hisArticle.getGhId());
                if (Objects.nonNull(firstIndexArticle) && firstIndexArticle.containsKey(hisArticle.getAppMsgId())) {
                    Article firstArticle = firstIndexArticle.get(hisArticle.getAppMsgId());
                    article.setFirstViewCount(firstArticle.getShowViewCount());
                    if (Objects.nonNull(firstIndexAvgInfo) && Objects.nonNull(firstIndexAvgInfo.getReadAvg())
                            && firstIndexAvgInfo.getReadAvg() > 0 && Objects.nonNull(firstArticle.getShowViewCount())) {
                        article.setFirstViewCountRate((firstArticle.getShowViewCount() * 1.0) / firstIndexAvgInfo.getReadAvg());
                    }
                }
                res.getHisPublishArticleList().add(article);
            }
            // 设置头条阅读均值
            result.put(cacheParam.getSourceId(), res);
        }
        return result;
    }

    private void setT0Data(Content content, String ghId) {
        if (CollectionUtils.isEmpty(content.getHisPublishArticleList())) {
            return;
        }
        Account account = accountRepository.getByGhId(ghId);
        int firstLevelSize = 0;
        int fissionSum = 0;
        double fissionWeightSum = 0;
        double fissionHeadWeightSum = 0;
        double fissionRecommendWeightSum = 0;
        int fansSum = 0;
        int avgReadCountSum = 0;
        Double t0FissionByFansSum = 0.0;
        Double t0FissionByReadAvgSum = 0.0;
        Double t0FissionByReadAvgCorrelationSum = 0.0;
        for (ContentHisPublishArticle article : content.getHisPublishArticleList()) {
            if (article.getItemIndex() != 1 || !article.isInnerAccount()) {
                continue;
            }
            if (Objects.nonNull(account) && !account.getType().equals(article.getAccountType())) {
                continue;
            }
            if (CollectionUtils.isEmpty(article.getArticleDetailInfoList())) {
                // 仅判断7.12以后发布文章
                if (article.getPublishTimestamp() > 1720713600 && contentHisFeishuEnable) {
                    log.error("历史表现裂变特征获取失败 ghId:{} accountName:{} itemIndex:{} title:{} date:{} wxsn:{}",
                            article.getGhId(), article.getAccountName(), article.getItemIndex(), article.getTitle(),
                            DateUtils.timestampToYMDStr(article.getPublishTimestamp(), "yyyyMMdd"), article.getWxSn());
//                    FeishuMessageSender.sendWebHookMessage(FeishuRobotIdEnum.RECOMMEND.getRobotId(),
//                            "历史表现裂变特征获取失败\n"
//                                    + "ghId: " + article.getGhId() + "\n"
//                                    + "账号名称: " + article.getAccountName() + "\n"
//                                    + "位置: " + article.getItemIndex() + "\n"
//                                    + "标题: " + article.getTitle() + "\n"
//                                    + "发布时间: " + DateUtils.timestampToYMDStr(article.getPublishTimestamp(), "yyyyMMdd") + "\n"
//                                    + "wxsn: " + article.getWxSn());
                }
                continue;
            }
            int sumFission0 = 0;
            int sumFission0Head = 0;
            int sumFission0Recommend = 0;
            Date minDate = article.getArticleDetailInfoList().stream().map(ArticleDetailInfo::getRecallDt)
                    .min(Date::compareTo).orElse(new Date());
            for (ArticleDetailInfo articleDetailInfo : article.getArticleDetailInfoList()) {
                if (articleDetailInfo.getRecallDt().equals(minDate)) {
                    if (Objects.nonNull(articleDetailInfo.getFission0())) {
                        sumFission0 += articleDetailInfo.getFission0();
                    }
                    if (Objects.nonNull(articleDetailInfo.getFission0Head())) {
                        sumFission0Head += articleDetailInfo.getFission0Head();
                    }
                    if (Objects.nonNull(articleDetailInfo.getFission0Recommend())) {
                        sumFission0Recommend += articleDetailInfo.getFission0Recommend();
                    }
                }
            }
            if (sumFission0 == 0) {
                continue;
            }
            Double correlation = (article.getCorrelation() + 1) / 2;
            article.setT0FissionSum(sumFission0);
            if (article.getFans() > 0) {
                article.setT0FissionByFans(sumFission0 * 1.0 / article.getFans());
                fansSum += (int) article.getFans();
                t0FissionByFansSum += article.getT0FissionByFans();
            }
            if (Objects.nonNull(article.getAvgViewCount()) && article.getAvgViewCount() > 0) {
                article.setT0FissionByReadAvg(sumFission0 * 1.0 / (article.getAvgViewCount() + 500));
                avgReadCountSum += article.getAvgViewCount();
                t0FissionByReadAvgSum += article.getT0FissionByReadAvg();
                t0FissionByReadAvgCorrelationSum += article.getT0FissionByReadAvg() * correlation;
            }
            fissionSum += sumFission0;
            int hour = DateUtils.getHourByTimestamp(article.getPublishTimestamp());
            if (hour >= 12) {
                fissionWeightSum += sumFission0 / morningNoonFissionRate;
                fissionHeadWeightSum += sumFission0Head / morningNoonFissionRate;
                fissionRecommendWeightSum += sumFission0Recommend / morningNoonFissionRate;
            } else {
                fissionWeightSum += sumFission0;
                fissionHeadWeightSum += sumFission0Head;
                fissionRecommendWeightSum += sumFission0Recommend;
            }
            firstLevelSize++;
        }
        if (firstLevelSize > 0) {
            content.setT0FissionByFansMean(t0FissionByFansSum / firstLevelSize);
            content.setT0FissionByReadAvgMean(t0FissionByReadAvgSum / firstLevelSize);
            content.setT0FissionByReadAvgCorrelationMean(t0FissionByReadAvgCorrelationSum / firstLevelSize);
            if (fansSum > 0) {
                content.setT0FissionByFansSumAvg(fissionSum * 1.0 / fansSum);
            }
            if (avgReadCountSum > 0) {
                content.setT0FissionByReadAvgSumAvg(fissionSum * 1.0 / avgReadCountSum);
            }
            if (avgReadCountSum > 0) {
                content.setT0FissionDeWeightByReadAvgSumAvg(fissionWeightSum / (avgReadCountSum + 500));
                content.setT0FissionHeadDeWeightByReadAvgSumAvg(fissionHeadWeightSum / (avgReadCountSum + 500));
                content.setT0FissionRecommendDeWeightByReadAvgSumAvg(fissionRecommendWeightSum / (avgReadCountSum + 500));
            }
        }
    }

    public static void main(String[] args) {
        String url = "http://mp.weixin.qq.com/s?__biz=Mzg2ODk4MTg3OQ==&mid=2247488306&idx=1&sn=93ebadc5bc7161a0dee48355013d3bc4&chksm=cfb6c1cb2bcdd80dd16d5d604d741a0019ae791125265a042d26100ba21ddb9e5c643ecc2264&scene=126&sessionid=1679649075#rd";
        String md5 = generateArticleUniqueMd5(url);
        System.out.println("Generated md5: " + md5);
    }

}
