package com.tzld.longarticle.recommend.server.service.recommend.score.strategy;

import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.service.recommend.config.AccountIndexAvgViewCountService;
import com.tzld.longarticle.recommend.server.service.recommend.score.Score;
import com.tzld.longarticle.recommend.server.service.recommend.score.ScoreParam;
import com.tzld.longarticle.recommend.server.service.recommend.score.ScoreStrategy;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

@Component
@Slf4j
public class HisFissionDeWeightAvgReadSumRateStrategy implements ScoreStrategy {

    @Autowired
    AccountIndexAvgViewCountService accountIndexAvgViewCountService;

    @ApolloJsonValue("${scoreFissionWeightConfig:{}}")
    private Map<String, Map<String, Double>> scoreFissionWeightConfig;

    @Override
    public List<Score> score(ScoreParam param) {
        List<Score> scores = new ArrayList<>();
        Map<String, Double> scoreFissionWeightConfigMap = scoreFissionWeightConfig.get(param.getPlanId());
        for (Content content : param.getContents()) {
            if (CollectionUtils.isEmpty(content.getHisPublishArticleList())) {
                continue;
            }
            Score score = new Score();
            score.setStrategy(this);
            score.setContentId(content.getId());
            Double scoreValue = 0.0;
            if (Objects.nonNull(scoreFissionWeightConfigMap)) {
                if (Objects.nonNull(content.getT0FissionHeadDeWeightByReadAvgSumAvg())) {
                    scoreValue += content.getT0FissionHeadDeWeightByReadAvgSumAvg() * scoreFissionWeightConfigMap.get("head");
                }
                if (Objects.nonNull(content.getT0FissionRecommendDeWeightByReadAvgSumAvg())) {
                    scoreValue += content.getT0FissionRecommendDeWeightByReadAvgSumAvg() * scoreFissionWeightConfigMap.get("recommend");
                }
            }
            if (scoreValue == 0.0) {
                scoreValue = content.getT0FissionDeWeightByReadAvgSumAvg();
            }
            score.setScore(scoreValue);
            scores.add(score);
        }
        return scores;
    }
}
